% Copyright 2018 John S Glover
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%
% 1. Redistributions of source code must retain the above copyright notice,
%    this list of conditions and the following disclaimer.
%
% 2. Redistributions in binary form must reproduce the above copyright notice,
%    this list of conditions and the following disclaimer in the documentation
%    and/or other materials provided with the distribution.
%
% 3. Neither the name of the copyright holder nor the names of its
%    contributors may be used to endorse or promote products derived from
%    this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
% NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
% STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
% ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\chapter{Makefile}
\label{Makefile}

A simple \verb|Makefile| is provided for constructing common targets.
This \verb|Makefile| uses the GNU gmake nomenclature~\cite{GNUMake2016}.
Most targets work with regular \verb|make|, but there are some limitations.

Example targets include:
\begin{description}
\item {\tt make ljfilt} - Build the {\tt ljfilt} executable.
\item {\tt make tar} - Build all files and package into a gzipped tar file.
\end{description}

\lstset{language=make}

@o Makefile -t
@{@< BSD 3-clause licence @>

@< Define commands for Makefile@>
TARGET = ljfilt
VERSION = 1.1

BIBS = ljfilt.bib

SRCS = ljfilt.w ljfilt_make.w $(BIBS)

DIST = Makefile $(TARGET) $(TARGET).pdf $(SRCS) $(BIBS)

@< Implicit rules definitions for ljfilt @>
@< Explicit rules definitions for ljfilt @>
@< Utility rules for ljfilt @>
@| BIBS DIST SRCS TARGET VERSION @}

To ensure the commands are executed are those expected then a series of variables are defined.
This may be excessive, but it prevents rogue applications being executed.
A future refinement could make use of a utility such as {\tt Autoconf} \cite{GNUAutoconf2012}.

@d Define commands for Makefile
@{CP = /bin/cp
MKDIR = /bin/mkdir
MV = /bin/mv  
RM = /bin/rm
LPR = /usr/bin/lpr
SED = /usr/bin/sed
TAR = /usr/bin/tar
BIBTEX = /usr/local/bin/bibtex
DVIPDF = /usr/local/bin/dvipdf
LATEX = /usr/local/bin/latex
PDFLATEX = /usr/local/bin/pdflatex
XDVI = /usr/local/bin/xdvi
@}

@d Implicit rules definitions for ljfilt
@{${TARGET}.tex: $(SRCS)
	nuweb -l $(TARGET)

%.dvi: $(TARGET).tex
	$(LATEX) $(TARGET).tex

%.pdf: $(TARGET).tex
	$(PDFLATEX) $(TARGET).tex
@}

Several targets are provided with explicit rules.

@d Explicit rules definitions for ljfilt
@{all:
	$(MAKE) $(TARGET).tex
	$(MAKE) $(TARGET)

tar: $(SRCS)
	$(MAKE) $(TARGET)
	$(MKDIR) $(TARGET)-$(VERSION)
	$(CP) -R $(DIST) $(TARGET)-$(VERSION)
	$(MV) $(TARGET)-$(VERSION)/$(TARGET).pdf \
	$(TARGET)-$(VERSION)/$(TARGET)-$(VERSION).pdf
	$(TAR) -zcf $(TARGET)-$(VERSION).tar.gz $(TARGET)-$(VERSION)
	$(RM) -rf $(TARGET)-$(VERSION)

$(TARGET): $(SRCS)
	nuweb -l $(TARGET)
	$(LATEX) $(TARGET)
	nuweb -l $(TARGET)
	$(BIBTEX) $(TARGET)
	$(LATEX) $(TARGET)
	$(LATEX) $(TARGET)
	$(DVIPDF) $(TARGET)
@}

A number of routines and utilities are provided to support the maintenance of
the ljfilt source code directory.
The preceding \verb|'-'| ignores errors in the command line (non-zero) exit
status to be ignored the command to continue.

@d Utility rules for ljfilt
@{clean:
	-$(RM) -f *.tex *.log *.dvi *~ *.blg *.brf *.bbl *.out *.toc *.aux *.gz *.pdf \#*

view:	$(TARGET).dvi
	$(XDVI) $(TARGET).dvi
@}
